%/*--------------------------------------------------------------------------
%  ----- Institute:   Ruhr-Universitaet-Bochum
%                     Institut fuer Neuroinformatik
%		     
%  ----- File:        inv_kin8.m
%  
%  ----- Author:      Ioannis Iossifidis
%                     iossifidis@neuroinformatik.ruhr-uni-bochum.de
%  ----- Date:        1999-12-06
%  ----- last change: 2000-07-01           
%  
%  ----- Description: This function calculates the inverse kinematics 
%                     of an 8 DOF manipulator, based on the closed form 
%                     solution for a 7DOF arm by P.Dahm and F. Joublin. 
%		      The 8 DOF manipulator are divided in a 7 DOF arm 
%                     and a 1 DOF body.
%                     
%  
%  ----- Copyright:   (c) Ioannis Iossifidis 2000. 
%                     
%  ----- Warranties:  Of course, there's NO WARRANTY OF ANY KIND :-)
%  ----- Credits:     
%--------------------------------------------------------------------------*/

%--------------------------------------------------------------------------*/
%  function input:
%  - [X Y Z] is the task position in Cartesian coordinates (in mm).
%  - eef_phi and eef_theta is the task orientation of the end-effector.
%  - bodyAng is the body angle and armAng is the redundancy angle.
%
%  function output:
%  7 module angels (in deg)
%--------------------------------------------------------------------------*/



function [phi1,phi2,phi3,phi4,phi5,phi6,phi7] = inv_kin8(X,Y,Z, ...
    eef_phi, ...
    eef_theta,armAng,bodyAng)


deg2rad=pi/180.0;
rad2deg=180/pi;

%Limb length in mm
L_korpus =225.0;
L_o  = 325.0;%oberarmteil
L_u  =310.0;%unterarm
L_eef=285;

ex=[1;0;0];
ez=[0;0;1];
ey=[0;1;0];

%calcutating the endeffector position with respect  to the body position
r_eef=Rz(-bodyAng*deg2rad)*[X;Y;Z]-[L_korpus;0;400];


eef_theta=pi+eef_theta*deg2rad;
eef_phi=-pi/2+eef_phi*deg2rad;

vec_eef=Rz(eef_phi-bodyAng*deg2rad)*Ry(eef_theta)*ex*L_eef;




r_h=r_eef-(vec_eef);


%%%%%%%%%%%%%%%%% Elbow Position
d=sqrt(r_h(1)^2+r_h(2)^2+r_h(3)^2);        %laenge von r_hand
h=sqrt(L_o^2-((L_o^2-L_u^2+d^2)/2.0/d)^2);  %Redundanzradius

c=(L_o^2-L_u^2+d^2)/2.0/d;
%%%%%%%%%%%%%

THETA=acos(r_h(1)/d);    %%wrist
PHI=atan2(r_h(3),r_h(2));%wrist

ed=Rx(PHI) * Rz(THETA) * Rx(armAng*deg2rad)*(ez); %ez88  
r_o=ed .*h  + c/d *r_h;
r_m= c/d  *r_h;

%%%%%%%%%%%%%%%% Elbow


%%%%%%%%%%%%%%% Joint 1&2
eh2=r_o(1)/L_o;
phi1=atan2(r_o(3),r_o(2));
phi2=acos(eh2);
%%%%%%%%%%%%%%% Joint 1&2



%%%%%%%%%%%%%%%%%%%%%%%%%%%% Joint 3&4
vec_ru=r_h-r_o;
vec_out=  Rz(-phi2)*Rx(-phi1)* vec_ru;
L_vec_out=sqrt(vec_out(1)^2+vec_out(2)^2+vec_out(3)^2);

eh3=vec_out(1)/L_u;

phi3=atan2(vec_out(3),vec_out(2));
phi4=acos(eh3);
%%%%%%%%%%%%%%%%%%%%%%%%%%%% Joint 3&4

%%%%%%%%%%%%%%%%%%%%%%%%%%%% Joint 5&6Inv_kin7
vec_eef_out=Rz(-phi4)*Rx(-phi3)*Rz(-phi2)*Rx(-phi1)*vec_eef;
eh4=vec_eef_out(1)/L_eef;

phi5=atan2(vec_eef_out(3),vec_eef_out(2));
phi6=acos(eh4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%% Joint 5&6



r_W=r_eef-r_h;


%%%%%%%%%%%%%%%Gripper paralle to ground
zAxis=[0,0,1];  
grip(1) =  r_W(2)*zAxis(3) - r_W(3)*zAxis(2);
grip(2) =  r_W(3)*zAxis(1) - r_W(1)*zAxis(3);%+
grip(3) =  r_W(1)*zAxis(2) - r_W(2)*zAxis(1);%+
%%%%%%%%%%%%%%%Gripper paralle to ...


grip_out=Rz(-phi6)*Rx(-phi5)*Rz(-phi4)*Rx(-phi3)*Rz(-phi2)*Rx(-phi1)*grip';

phi7=atan2(grip_out(3),grip_out(2))+90*deg2rad;

